@@ -1,4 +1,4 @@
-# Copyright (c) 2025 OpenStack Foundation
+# Copyright (c) 2025 Thomas Goirand <zigo@debian.org>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,9 +26,11 @@ from oslo_log import log as logging
 from openstack import connection
 from keystoneauth1 import loading as ks_loading
 
-from vmms.db.sqlalchemy.database import get_session
+from vmms.db.sqlalchemy.database import init_database_config
+from vmms.db.sqlalchemy.database import get_session as get_db_session
 from vmms.db.sqlalchemy.models import VMMigration, MigrationState
 from vmms.mistral_client import MistralClient
+from vmms.policy import enforcer as policy_enforcer
 
 # Initialize logging
 LOG = logging.getLogger(__name__)
@@ -78,14 +80,6 @@ def get_config():
         _CONFIG = config.init_config()
     return _CONFIG
 
-def is_authenticated():
-    """Check if request is authenticated by Keystone middleware"""
-    # Check the X-Identity-Status header set by keystonemiddleware
-    identity_status = request.headers.get('X-Identity-Status')
-    if identity_status and identity_status.upper() == 'CONFIRMED':
-        return True
-    return False
-
 def get_vm_details_from_source_cloud(vm_identifier):
     # Get configuration (reuse existing config)
     CONF = get_config()
@@ -132,17 +126,28 @@ def get_vm_details_from_source_cloud(vm_identifier):
 # Create single Flask app with all routes
 app = Flask(__name__)
 
+# wrapper decorator, so we can pass the CONF
+# object ot the vmms/policy/enforcer.py
+def require_vmms_policy(action):
+    def decorator(f):
+        @wraps(f)
+        def wrapper(*args, **kwargs):
+            CONF = get_config()
+            return policy_enforcer.require_policy(CONF, action)(f)(*args, **kwargs)
+        return wrapper
+    return decorator
+
+policy_checker = policy_enforcer.require_policy_factory(get_config)
+
 @app.route('/healthcheck', methods=['GET'])
 def health_check():
     """Health check endpoint - no authentication required"""
     return jsonify({'status': 'ok', 'service': 'vmms'}), 200
 
 @app.route('/v2/vms', methods=['POST'])
+@policy_checker('vmms:add')
 def add_vm():
     """Add a VM to migration scheduler - authentication required"""
-    if not is_authenticated():
-        return jsonify({'error': 'Authentication required'}), 401
-        
     try:
         data = request.get_json()
         
@@ -165,7 +170,7 @@ def add_vm():
         LOG.info(f"Found VM: {vm_name} ({vm_id})")
         
         
-        session = get_session()
+        session = get_db_session()
         migration = VMMigration()
         migration.id = str(uuid.uuid4())
         migration.vm_id = vm_id
@@ -183,11 +188,9 @@ def add_vm():
         session.commit()
         
         result = migration.to_dict()
-        session.close()
         return jsonify(result), 201
     except IntegrityError as e:
         session.rollback()
-        session.close()
         if 'uniq_vm_id' in str(e) or 'vm_id' in str(e).lower():
            return jsonify({'error': f'VM with ID {vm_id} is already scheduled for migration'}), 409
         return jsonify({'error': 'Database integrity error'}), 500
@@ -195,15 +198,12 @@ def add_vm():
         LOG.error(f"Error adding VM: {e}", exc_info=True)
         if 'session' in locals():
             session.rollback()
-            session.close()
         return jsonify({'error': str(e)}), 500
 
 @app.route('/v2/vms', methods=['GET'])
+@policy_checker('vmms:list')
 def list_vms():
     """List all VM migrations - authentication required"""
-    if not is_authenticated():
-        return jsonify({'error': 'Authentication required'}), 401
-        
     try:
         # Get query parameters
         state_filter = request.args.get('state', None)
@@ -217,7 +217,7 @@ def list_vms():
                     'error': f'Invalid state. Allowed: {", ".join(valid_states)}'
                 }), 400
 
-        session = get_session()
+        session = get_db_session()
 
         # Build query with optional state filtering
         query = session.query(VMMigration)
@@ -225,35 +225,28 @@ def list_vms():
             query = query.filter(VMMigration.state == state_filter)
         migrations = query.all()
         result = [migration.to_dict() for migration in migrations]
-        session.close()
         return jsonify(result)
     except Exception as e:
         LOG.error(f"Error listing VMs: {e}", exc_info=True)
-        if 'session' in locals():
-            session.close()
         return jsonify({'error': str(e)}), 500
 
 @app.route('/v2/vms/<string:migration_id>', methods=['DELETE'])
+@policy_checker('vmms:remove')
 def remove_vm(migration_id):
     """Remove a VM from migration scheduler - authentication required"""
-    if not is_authenticated():
-        return jsonify({'error': 'Authentication required'}), 401
-
     # Validate migration_id format
     if not is_valid_uuid(migration_id):
         return jsonify({'error': 'Invalid migration ID format. Must be a valid UUID.'}), 400
 
     try:
-        session = get_session()
+        session = get_db_session()
         migration = session.query(VMMigration).filter_by(id=migration_id).first()
         
         if not migration:
-            session.close()
             return jsonify({'error': f'Migration with ID {migration_id} not found'}), 404
             
         session.delete(migration)
         session.commit()
-        session.close()
         
         return jsonify({'message': 'Migration removed successfully'}), 200
         
@@ -261,44 +254,38 @@ def remove_vm(migration_id):
         LOG.error(f"Error removing VM: {e}", exc_info=True)
         if 'session' in locals():
             session.rollback()
-            session.close()
         return jsonify({'error': str(e)}), 500
 
 
 @app.route('/v2/vms/<string:migration_id>', methods=['GET'])
+@policy_checker('vmms:show')
 def show_vm(migration_id):
     """Show details of a specific VM migration - authentication required"""
-    if not is_authenticated():
-        return jsonify({'error': 'Authentication required'}), 401
-
     # Validate migration_id format
     if not is_valid_uuid(migration_id):
         return jsonify({'error': 'Invalid migration ID format. Must be a valid UUID.'}), 400
         
     try:
-        session = get_session()
+        session = get_db_session()
         migration = session.query(VMMigration).filter_by(id=migration_id).first()
         
         if not migration:
-            session.close()
             return jsonify({'error': f'Migration with ID {migration_id} not found'}), 404
             
         result = migration.to_dict()
-        session.close()
         return jsonify(result), 200
         
     except Exception as e:
         LOG.error(f"Error showing VM: {e}", exc_info=True)
-        if 'session' in locals():
-            session.close()
         return jsonify({'error': str(e)}), 500
 
 
 @app.route('/v2/vms/<string:migration_id>', methods=['PUT'])
+@policy_checker('vmms:update')
 def update_vm(migration_id):
     """Update a VM migration - authentication required"""
-    if not is_authenticated():
-        return jsonify({'error': 'Authentication required'}), 401
+    LOG.debug(f"🐛 API: update_vm called by user with migration_id: {migration_id}")
+    LOG.debug(f"🐛 DEBUG: This should NEVER be reached if policy enforcement works!")
 
     # Validate migration_id format
     if not is_valid_uuid(migration_id):
@@ -328,18 +315,16 @@ def update_vm(migration_id):
                 return jsonify({'error': 'Invalid scheduled_time format. Must be ISO 8601 format.'}), 400
         
         
-        session = get_session()
+        session = get_db_session()
         migration = session.query(VMMigration).filter_by(id=migration_id).first()
         
         if not migration:
-            session.close()
             return jsonify({'error': f'Migration with ID {migration_id} not found'}), 404
             
         # Handle scheduled_time updates
         if 'scheduled_time' in data:
             # Allow unsetting scheduled_time for any state, but setting it only for scheduled migrations
             if data['scheduled_time'] is not None and migration.state != MigrationState.SCHEDULED:
-                session.close()
                 return jsonify({'error': 'Can only update scheduled_time for scheduled migrations'}), 400
                 
             try:
@@ -348,7 +333,6 @@ def update_vm(migration_id):
                 else:
                     migration.scheduled_time = None
             except ValueError:
-                session.close()
                 return jsonify({'error': 'Invalid scheduled_time format'}), 400
         
         # Handle state updates
@@ -356,7 +340,6 @@ def update_vm(migration_id):
             # Get all possible state values from the enum
             allowed_states = [state.value for state in MigrationState]
             if data['state'] not in allowed_states:
-                session.close()
                 return jsonify({'error': f'Invalid state. Allowed: {", ".join(allowed_states)}'}), 400
             migration.state = data['state']
 
@@ -366,31 +349,25 @@ def update_vm(migration_id):
 
         # Validate that workflow_exec is not being set via API
         if 'workflow_exec' in data:
-            session.close()
             return jsonify({'error': 'workflow_exec is read-only and cannot be set via API'}), 400
         
         session.commit()
         result = migration.to_dict()
-        session.close()
         return jsonify(result), 200
         
     except Exception as e:
         LOG.error(f"Error updating VM: {e}", exc_info=True)
         if 'session' in locals():
             session.rollback()
-            session.close()
         return jsonify({'error': str(e)}), 500
 
 
 @app.route('/v2/vms/<string:migration_id>/output', methods=['GET'])
+@policy_checker('vmms:output')
 def get_vm_output(migration_id):
     """Get output from VM migration - authentication required"""
     LOG.debug(f"🐛 GET /v2/vms/{migration_id}/output called")
 
-    if not is_authenticated():
-        LOG.debug("🐛 Authentication failed")
-        return jsonify({'error': 'Authentication required'}), 401
-
     # Validate migration_id format
     if not is_valid_uuid(migration_id):
         LOG.debug(f"🐛 Invalid UUID format: {migration_id}")
@@ -398,17 +375,15 @@ def get_vm_output(migration_id):
 
     try:
         # Get database session
-        session = get_session()
+        session = get_db_session()
         migration = session.query(VMMigration).filter_by(id=migration_id).first()
 
         if not migration:
-            session.close()
             LOG.debug(f"🐛 Migration not found: {migration_id}")
             return jsonify({'error': f'Migration with ID {migration_id} not found'}), 404
 
         # Check if migration has a workflow execution
         if not migration.workflow_exec:
-            session.close()
             LOG.debug(f"🐛 No workflow execution for migration: {migration_id}")
             return jsonify({'error': f'Migration {migration_id} has no workflow execution'}), 404
 
@@ -423,15 +398,12 @@ def get_vm_output(migration_id):
         LOG.debug(f"🐛 State check result: {migration.state in allowed_states}")
 
         if migration.state not in allowed_states:
-            session.close()
             LOG.debug(f"🐛 Invalid state for output retrieval: {migration.state}")
             return jsonify({
                 'error': f'Cannot retrieve output for migration in state {migration.state}. '
                          f'Output is only available for MIGRATED or ERROR states.'
             }), 400
 
-        session.close()
-
         # Get Mistral client and retrieve output
         CONF = get_config()
         from vmms.mistral_client import MistralClient
@@ -445,8 +417,6 @@ def get_vm_output(migration_id):
         
     except Exception as e:
         LOG.error(f"⧱ Error getting VM output for migration {migration_id}: {e}", exc_info=True)
-        if 'session' in locals():
-            session.close()
         return jsonify({'error': f'Failed to retrieve migration output: {str(e)}'}), 500
 
 
@@ -456,6 +426,9 @@ def get_application():
         CONF = get_config()
         logging.setup(CONF, 'vmms-api')
         
+        # Initialize database configuration
+        init_database_config(CONF)
+
         # Apply Keystone middleware with delay_auth_decision=true
         from keystonemiddleware import auth_token
         middleware_conf = dict(CONF.keystone_authtoken)
