@@ -1,4 +1,4 @@
-# Copyright (c) 2025 OpenStack Foundation
+# Copyright (c) 2025 Thomas Goirand <zigo@debian.org>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,7 +27,8 @@ import tooz
 from tooz import coordination
 
 from vmms import config
-from vmms.db.sqlalchemy.database import get_session
+from vmms.db.sqlalchemy.database import get_session as get_db_session
+from vmms.db.sqlalchemy.database import init_database_config
 from vmms.db.sqlalchemy.models import VMMigration, MigrationState
 from vmms.mistral_client import MistralClient
 
@@ -126,6 +127,9 @@ class WorkerService(object):
         LOG.info(f"⚙ VMMS worker is starting up...")
         LOG.info(f"⚙ VMMS worker Service (node_id: {self.CONF.worker.my_hostname})")
 
+        # Initialize database configuration
+        init_database_config(self.CONF)
+
         if not self.initialize_coordination():
             LOG.error(f"⧱ Failed to initialize coordination, exiting")
             return False
@@ -303,7 +307,7 @@ class WorkerService(object):
         """Monitor MIGRATING migrations and update their status."""
         LOG.debug(f"🐛 Monitoring running migrations")
 
-        session = get_session()
+        session = get_db_session()
         try:
             # Find all migrations in MIGRATING state that we own
             migrating_migrations = session.query(VMMigration).filter(
@@ -318,7 +322,7 @@ class WorkerService(object):
                     LOG.error(f"⧱ Error checking migration {migration.id}: {e}")
 
         finally:
-            session.close()
+            pass  # No need to close session with oslo.db
 
     def check_and_update_migration_status(self, session, migration):
         """Check workflow execution status and update migration state."""
@@ -421,27 +425,24 @@ class WorkerService(object):
         current_hour = current_datetime.hour
         LOG.debug(f"🐛 Processing scheduled migrations for {current_date} hour {current_hour}, slots available: {slots_available}")
 
-        session = get_session()
-        try:
-            # Find SCHEDULED migrations for current date and hour, limited to available slots
-            # Compare date and hour separately to ignore minutes/seconds
-            scheduled_migrations = session.query(VMMigration).filter(
-                VMMigration.state == MigrationState.SCHEDULED,
-                VMMigration.scheduled_time.isnot(None),
-                func.date(VMMigration.scheduled_time) == current_date,
-                func.extract('hour', VMMigration.scheduled_time) == current_hour
-            ).limit(slots_available).all()
+        session = get_db_session()
 
-            processed_count = 0
-            for migration in scheduled_migrations:
-                if self.trigger_migration_workflow(session, migration):
-                    processed_count += 1
-                        
-            LOG.info(f"⚙ Processed {processed_count} scheduled migrations")
-            return slots_available - processed_count
-            
-        finally:
-            session.close()
+        # Find SCHEDULED migrations for current date and hour, limited to available slots
+        # Compare date and hour separately to ignore minutes/seconds
+        scheduled_migrations = session.query(VMMigration).filter(
+            VMMigration.state == MigrationState.SCHEDULED,
+            VMMigration.scheduled_time.isnot(None),
+            func.date(VMMigration.scheduled_time) == current_date,
+            func.extract('hour', VMMigration.scheduled_time) == current_hour
+        ).limit(slots_available).all()
+
+        processed_count = 0
+        for migration in scheduled_migrations:
+            if self.trigger_migration_workflow(session, migration):
+                processed_count += 1
+                    
+        LOG.info(f"⚙ Processed {processed_count} scheduled migrations")
+        return slots_available - processed_count
 
     def process_opportunistic_migrations(self, slots_available):
         """Process opportunistic migrations within time window.
@@ -451,23 +452,20 @@ class WorkerService(object):
         """
         LOG.debug(f"🐛 Processing opportunistic migrations, slots available: {slots_available}")
 
-        session = get_session()
-        try:
-            # Find any SCHEDULED migrations without specific schedule time, limited to available slots
-            opportunistic_migrations = session.query(VMMigration).filter(
-                VMMigration.state == MigrationState.SCHEDULED,
-                VMMigration.scheduled_time.is_(None)
-            ).limit(slots_available).all()
-            
-            processed_count = 0
-            for migration in opportunistic_migrations:
-                if self.trigger_migration_workflow(session, migration):
-                    processed_count += 1
-                    
-            LOG.info(f"⚙ Processed {processed_count} opportunistic migrations")
-            
-        finally:
-            session.close()
+        session = get_db_session()
+
+        # Find any SCHEDULED migrations without specific schedule time, limited to available slots
+        opportunistic_migrations = session.query(VMMigration).filter(
+            VMMigration.state == MigrationState.SCHEDULED,
+            VMMigration.scheduled_time.is_(None)
+        ).limit(slots_available).all()
+        
+        processed_count = 0
+        for migration in opportunistic_migrations:
+            if self.trigger_migration_workflow(session, migration):
+                processed_count += 1
+
+        LOG.info(f"⚙ Processed {processed_count} opportunistic migrations")
 
     def is_within_migration_window(self):
         """Check if current time is within allowed migration window and weekday."""
